@@ -310,8 +310,8 @@ class ApiBase:
         try:
             async for result in processed_results:
                 count += 1
-                if result is None:
-                    self.logger.warning(f'Results is from `fetcher`is None. Ignore and continue')
+                # if result is None:
+                #     self.logger.warning(f'Results from `fetcher`is None. Ignore and continue')
                 #     continue
 
                 results_to_save.append(result)
@@ -322,16 +322,23 @@ class ApiBase:
                     self.logger.info(f'Processed {count} so far. Save interval of {save_interval} reached.')
                     if saver:
                         self.logger.info(f'Saving {len(results_to_save)} results')
-                        data_to_save = transformer(results_to_save)
-                        saver(data_to_save)
+                        data_to_save = await asyncio.to_thread(transformer,results_to_save)
+                        await asyncio.to_thread(saver,data_to_save)
                     all_results.extend(results_to_save)
                     results_to_save.clear()
+        except (RateLimitError, APIkeyError, PermissionError) as fatal_errors:
+            self.logger.error(f'Fatal error in `_process_tasks`, stopping process: {fatal_errors}')
+            raise
+        except SkipItemException as item_error:
+            self.logger.warning(f'Skipping item due to a recoverable error: {item_error}')
+        except Exception as e:
+            self.logger.error(f"An unexpected error occurred in task processing loop: {e}")
 
         finally:
             if results_to_save:
                 if saver:
-                    data_to_save = transformer(results_to_save)
-                    saver(data_to_save)
+                    data_to_save = await asyncio.to_thread(transformer, results_to_save)
+                    await asyncio.to_thread(saver, data_to_save)
                 all_results.extend(results_to_save)
                 results_to_save.clear()
 
@@ -342,9 +349,10 @@ class ApiBase:
                                inputs : list | dict,
                                fetcher : Callable,
                                transformer : Callable[[List],any],
-                               saver : Callable,
+                               saver : Callable = None,
                                save_interval: int = 50000,
                                concurrent_requests : int = 5,
+                               return_result : bool = False,
                                ) -> list:
         """
         Fetches multiple items concurrently, associating each result with a unique ID.
@@ -357,8 +365,8 @@ class ApiBase:
             inputs (list | dict): A list of items or a dictionary where keys are item IDs
                                    and values are the items to be fetched.
             fetcher (callable): An asynchronous function that takes a single item as input and fetches its data.
-            transformer (callable): A function to transform the results.
-            saver (callable): A function to save the results.
+            transformer (callable): A function to transform the results. NB: Must take a list as input!
+            saver (callable): A function to save the results. Optional, if not provided, data will not be saved.
             save_interval (int): The number of results to accumulate before saving (if `save` is True).
             concurrent_requests (int): The maximum number of concurrent API requests.
 
@@ -411,16 +419,18 @@ class ApiBase:
         tasks = [fetch_item_with_id(item_id=item_id, item=item) for item_id,item in inputs.items()]
 
         all_results = await self._process_tasks(tasks = tasks, transformer = transformer, saver = saver, save_interval=save_interval)
-        output = transformer(all_results)
-        return output
+        if return_result:
+            output = transformer(all_results)
+            return output
 
     async def get_items(self,
                                inputs : list | dict,
                                fetcher : Callable,
                                transformer : Callable[[List],any],
-                               saver : Callable,
+                               saver : Callable = None,
                                save_interval: int = 50000,
                                concurrent_requests : int = 5,
+                               return_result: bool = False,
                                ) -> list:
         """
         Fetches multiple items concurrently.
@@ -431,8 +441,8 @@ class ApiBase:
         Args:
             inputs (list): A list of items to be fetched.
             fetcher (callable): An asynchronous function that takes a single item as input and fetches its data.
-            transformer (callable): A function to transform the results.
-            saver (callable): A function to save the results.
+            transformer (callable): A function to transform the results. NB: Must take a list as input!
+            saver (callable): A function to save the results. Optional, if not provided, data will not be saved.
             save_interval (int): The number of results to accumulate before saving (if `save` is True).
             concurrent_requests (int): The maximum number of concurrent API requests.
 
@@ -472,5 +482,6 @@ class ApiBase:
         tasks = [fetch_item(item=item) for item in inputs]
 
         all_results = await self._process_tasks(tasks=tasks, transformer=transformer, saver=saver,save_interval=save_interval)
-        output = transformer(all_results)
-        return output
+        if return_result:
+            output = transformer(all_results)
+            return output
